/*
 * (c) 2006 Red Hat, Inc.
 *
 * This program is open source software licensed under the 
 * Eclipse Public License ver. 1
 */
package com.redhat.eclipse.changelog.core.parsers;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jdt.ui.IWorkingCopyManager;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;

import com.redhat.eclipse.changelog.core.IParserChangeLogContrib;

/**
 * @author pmuldoon (Phil Muldoon)
 */
public class JavaParser implements IParserChangeLogContrib {

	/**
	 * @see IParserChangeLogContrib#parseCurrentFunction(IEditorPart)
	 */
	public String parseCurrentFunction(IEditorInput input, int offset)
			throws CoreException {
		IWorkingCopyManager manager = JavaUI.getWorkingCopyManager();
		manager.connect(input);

		ICompilationUnit workingCopy = manager.getWorkingCopy(input);
		IJavaElement method = workingCopy.getElementAt(offset);
		manager.disconnect(input);

		// no element selected
		if (method == null)
			return "";

		String currentElementName = "";

		if ((currentElementName = method.getElementName()) == null) {
			// element doesn't have a name
			return "";
		}

		int elementType = method.getElementType();

		switch (elementType) {
		case IJavaElement.METHOD:
		case IJavaElement.FIELD:
			break;
		case IJavaElement.COMPILATION_UNIT:
			return "";
		default:
			IJavaElement tmpMethodType;
			if (((tmpMethodType = method.getAncestor(IJavaElement.METHOD)) == null)
					&& ((tmpMethodType = method.getAncestor(IJavaElement.TYPE)) == null)) {
				return "";
			} else {
				// cursor is inside a class, but not method
				method = tmpMethodType;
				currentElementName = method.getElementName();
			}
		}

		// now append all ancestor class names to string

		IJavaElement tmpParent = method.getParent();
		boolean firstLoop = true;

		while (tmpParent != null) {
			IJavaElement tmpParentClass = tmpParent
					.getAncestor(IJavaElement.TYPE);
			if (tmpParentClass != null) {
				String tmpParentClassName = tmpParentClass.getElementName();
				if (tmpParentClassName == null)
					return "";
				currentElementName = tmpParentClassName + "."
						+ currentElementName;
			} else {
				// cut root class name
				int rootClassPos = currentElementName.indexOf(".");
				if (rootClassPos >= 0)
					currentElementName = currentElementName
							.substring(rootClassPos + 1);
				if (firstLoop)
					return "";
				else
					return currentElementName;
			}
			tmpParent = tmpParentClass.getParent();
			firstLoop = false;

		}

		return "";
	}

	/**
	 * @see IParserChangeLogContrib#parseCurrentFunction(IEditorInput, int)
	 */
	public String parseCurrentFunction(IEditorPart editor) throws CoreException {

		// Check for type casting
		if (!(editor instanceof CompilationUnitEditor))
			return "";

		CompilationUnitEditor java_editor = (CompilationUnitEditor) editor;
		ITextSelection selection = (ITextSelection) java_editor
				.getSelectionProvider().getSelection();

		IEditorInput input = java_editor.getEditorInput();

		return parseCurrentFunction(input, selection.getOffset());
	}

}
